@@ -3,6 +3,7 @@
 
 #include<stdlib.h>
 #include<pabidecl.h>
+#include "pstring.h"
 
 typedef struct pbuffer {
   size_t bufsize;
@@ -12,11 +13,14 @@ typedef struct pbuffer {
 #define PBUFFER_MULTIPLICATOR 2
 
 TMPLPRO_LOCAL size_t pbuffer_size(pbuffer*);
+TMPLPRO_LOCAL void   pbuffer_preinit(pbuffer* pBuffer);
 TMPLPRO_LOCAL char*  pbuffer_init(pbuffer*);
 TMPLPRO_LOCAL char*  pbuffer_init_as(pbuffer* pBuffer,size_t size);
 TMPLPRO_LOCAL char*  pbuffer_string(pbuffer*);
 TMPLPRO_LOCAL char*  pbuffer_resize(pbuffer*, size_t size);
 TMPLPRO_LOCAL void   pbuffer_free(pbuffer*);
+// currently unused
+TMPLPRO_LOCAL void pbuffer_fill_from_pstring(pbuffer* pBuffer, PSTRING pstr);
 
 #endif /* pbuffer.h */
 
