@@ -2,10 +2,10 @@
 #define _TAGSTACK_H	1
 
 struct tagstack_entry {
-  int tag;
-  int value;
-  int vcontext;
-  char* position;
+  int tag;		/* code of tag */
+  int value;		/* if (true/false) - used in else */
+  int vcontext;		/* visibility context of the tag (visible/unvisible) */
+  char* position;	/* start of tag; useful for loops */
 };
 
 static 
@@ -17,19 +17,14 @@ void tagstack_push(struct tagstack* tagstack, struct tagstack_entry);
 static 
 struct tagstack_entry tagstack_pop(struct tagstack* tagstack);
 static 
-struct tagstack_entry tagstack_top(struct tagstack* tagstack);
-static 
-struct tagstack_entry* tagstack_head(struct tagstack* tagstack);
+struct tagstack_entry* tagstack_top(struct tagstack* tagstack);
 static 
 int tagstack_notempty(struct tagstack* tagstack);
-static 
-int tagstack_selftest(struct tagstack* tagstack);
 
 #endif /* tagstack.h */
 
 /*
  * Local Variables:
  * mode: c 
- * coding: cp1251
  * End: 
  */
